@@ -1,3 +1,4 @@
+/* $OpenLDAP: pkg/ldap/clients/gopher/go500gw.c,v 1.27.2.5 2001/07/21 19:01:37 kurt Exp $ */
 /*
  * Copyright (c) 1990 Regents of the University of Michigan.
  * All rights reserved.
@@ -13,7 +14,8 @@
 #include "portable.h"
 
 #include <stdio.h>
-#include <stdlib.h>
+
+#include <ac/stdlib.h>
 
 #include <ac/ctype.h>
 #include <ac/signal.h>
@@ -24,26 +26,28 @@
 #include <ac/unistd.h>
 #include <ac/wait.h>
 
+#include <ac/param.h>
 #include <ac/setproctitle.h>
 
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
-
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
 
 
-#include "lber.h"
-#include "ldap.h"
+#include <ldap.h>
+#include <disptmpl.h>
+
+#define ldap_debug debug
+#include "ldap_log.h"
+
 #include "lutil.h"
 
-#include "disptmpl.h"
 
-#include "ldapconfig.h"
+#include "ldap_defaults.h"
 
-int	debug = 0;
+int	debug;
+int ldap_syslog;
+int ldap_syslog_level;
 int	dosyslog;
 int	inetd;
 int	dtblsize;
@@ -57,10 +61,10 @@ char		*templatefile = TEMPLATEFILE;
 char		*friendlyfile = FRIENDLYFILE;
 int		rdncount = GO500GW_RDNCOUNT;
 
-static void usage	( char *name );
+static void usage	( char *name ) LDAP_GCCATTR((noreturn));
 static int  set_socket	(int port);
 static RETSIGTYPE wait4child(int sig);
-static void do_queries	(int s);
+static void do_queries	(int s)    LDAP_GCCATTR((noreturn));
 static char *pick_oc	( char **oclist );
 static int  isnonleaf	( LDAP *ld, char **oclist, char *dn );
 static void do_menu	(LDAP *ld, FILE *fp, char *dn);
@@ -68,7 +72,7 @@ static void do_list	(LDAP *ld, FILE *fp, char *dn);
 static int  isoc	( char **ocl, char *oc );
 static int  make_scope	( LDAP *ld, char *dn );
 static void do_search	(LDAP *ld, FILE *fp, char *query);
-static int  entry2textwrite( void *fp, char *buf, int len );
+static int  entry2textwrite( void *fp, char *buf, ber_len_t len );
 static void do_read	(LDAP *ld, FILE *fp, char *dn);
 static void do_help	(FILE *op);
 static void do_sizelimit(FILE *fp, char type);
@@ -81,7 +85,7 @@ static void
 usage( char *name )
 {
 	fprintf( stderr, "usage: %s [-d debuglevel] [-I] [-p port] [-P ldapport] [-l]\r\n\t[-x ldaphost] [-a] [-h helpfile] [-f filterfile] [-t templatefile] [-c rdncount]\r\n", name );
-	exit( 1 );
+	exit( EXIT_FAILURE );
 }
 
 int
@@ -94,7 +98,7 @@ main (int  argc, char **argv )
 	fd_set			readfds;
 	struct hostent		*hp;
 	struct sockaddr_in	from;
-	int			fromlen;
+	socklen_t         	fromlen;
 
 #if defined( LDAP_PROCTITLE ) && !defined( HAVE_SETPROCTITLE )
 	/* for setproctitle */
@@ -110,9 +114,6 @@ main (int  argc, char **argv )
 
 		case 'd':	/* debugging level */
 			debug |= atoi( optarg );
-#ifndef LDAP_DEBUG
-			fprintf( stderr, "warning: ldap debugging requires LDAP_DEBUG\n" );
-#endif
 			break;
 
 		case 'f':	/* ldap filter file */
@@ -175,11 +176,12 @@ main (int  argc, char **argv )
 #ifdef GO500GW_HOSTNAME
 	strcpy( myhost, GO500GW_HOSTNAME );
 #else
-	if ( myhost[0] == '\0' && gethostname( myhost, sizeof(myhost) )
+	if ( myhost[0] == '\0' && gethostname( myhost, sizeof(myhost)-1 )
 	    == -1 ) {
 		perror( "gethostname" );
-		exit( 1 );
+		exit( EXIT_FAILURE );
 	}
+	myhost[sizeof(myhost)-1] = '\0';
 #endif
 
 	/* detach if stderr is redirected or no debugging */
@@ -192,7 +194,8 @@ main (int  argc, char **argv )
 		myname = strdup( myname + 1 );
 
 	if ( debug ) {
-		lber_debug = ldap_debug = debug;
+		ber_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &debug);
+		ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &debug);
 	}
 	
 #ifdef SIGPIPE
@@ -202,7 +205,7 @@ main (int  argc, char **argv )
 	if ( dosyslog ) {
 #ifdef LOG_LOCAL3
 		openlog( myname, OPENLOG_OPTIONS, LOG_LOCAL3 );
-#else
+#elif LOG_DEBUG
 		openlog( myname, OPENLOG_OPTIONS );
 #endif
 	}
@@ -234,16 +237,16 @@ main (int  argc, char **argv )
 			}
 
 #ifdef LDAP_PROCTITLE
-			setproctitle( hp == NULL ? inet_ntoa( from.sin_addr ) :
+			setproctitle( "%s", hp == NULL ? inet_ntoa( from.sin_addr ) :
 			    hp->h_name );
 #endif
 		}
 
 		do_queries( 0 );
 
-		close( 0 );
+		tcp_close( 0 );
 
-		exit( 0 );
+		exit( EXIT_SUCCESS );
 	}
 
 	for ( ;; ) {
@@ -264,7 +267,7 @@ main (int  argc, char **argv )
 		if ( (ns = accept( s, (struct sockaddr *) &from, &fromlen ))
 		    == -1 ) {
 			if ( debug ) perror( "accept" );
-			exit( 1 );
+			exit( EXIT_FAILURE );
 		}
 
 		hp = gethostbyaddr( (char *) &(from.sin_addr),
@@ -278,7 +281,7 @@ main (int  argc, char **argv )
 
 		switch( pid = fork() ) {
 		case 0:		/* child */
-			close( s );
+			tcp_close( s );
 			do_queries( ns );
 			break;
 
@@ -287,7 +290,7 @@ main (int  argc, char **argv )
 			break;
 
 		default:	/* parent */
-			close( ns );
+			tcp_close( ns );
 			if ( debug )
 				fprintf( stderr, "forked child %d\n", pid );
 			break;
@@ -308,30 +311,41 @@ set_socket( int port )
 
 	if ( (s = socket( AF_INET, SOCK_STREAM, 0 )) == -1 ) {
                 perror( "socket" );
-                exit( 1 );
+                exit( EXIT_FAILURE );
         }
 
+#ifdef SO_REUSEADDR
         /* set option so clients can't keep us from coming back up */
-	one = 1;
+		one = 1;
         if ( setsockopt( s, SOL_SOCKET, SO_REUSEADDR, (char *) &one,
 	    sizeof(one) ) < 0 ) {
                 perror( "setsockopt" );
-                exit( 1 );
+                exit( EXIT_FAILURE );
         }
+#endif
+#ifdef SO_KEEPALIVE
+		/* enable keep alives */
+		one = 1;
+        if ( setsockopt( s, SOL_SOCKET, SO_KEEPALIVE, (char *) &one,
+	    sizeof(one) ) < 0 ) {
+                perror( "setsockopt" );
+                exit( EXIT_FAILURE );
+        }
+#endif
 
         /* bind to a name */
         addr.sin_family = AF_INET;
-        addr.sin_addr.s_addr = INADDR_ANY;
+        addr.sin_addr.s_addr = htonl(INADDR_ANY);
         addr.sin_port = htons( port );
         if ( bind( s, (struct sockaddr *) &addr, sizeof(addr) ) ) {
                 perror( "bind" );
-                exit( 1 );
+                exit( EXIT_FAILURE );
         }
 
 	/* listen for connections */
         if ( listen( s, 5 ) == -1 ) {
                 perror( "listen" );
-                exit( 1 );
+                exit( EXIT_FAILURE );
         }
 
         if ( debug )
@@ -357,7 +371,7 @@ wait4child( int sig )
 		;	/* NULL */
 #endif
 
-	(void) SIGNAL( SIGCHLD, wait4child );
+	(void) SIGNAL_REINSTALL ( SIGCHLD, wait4child );
 }
 
 static void
@@ -367,13 +381,14 @@ do_queries( int s )
 	int		len;
 	FILE		*fp;
 	int		rc;
+	int		deref;
 	struct timeval	timeout;
 	fd_set		readfds;
 	LDAP		*ld;
 
 	if ( (fp = fdopen( s, "a+")) == NULL ) {
 		perror( "fdopen" );
-		exit( 1 );
+		exit( EXIT_FAILURE );
 	}
 
 	timeout.tv_sec = GO500GW_TIMEOUT;
@@ -382,16 +397,16 @@ do_queries( int s )
 	FD_SET( fileno( fp ), &readfds );
 
 	if ( (rc = select( dtblsize, &readfds, 0, 0, &timeout )) <= 0 )
-		exit( 1 );
+		exit( EXIT_FAILURE );
 
 	if ( fgets( buf, sizeof(buf), fp ) == NULL )
-		exit( 1 );
+		exit( EXIT_FAILURE );
 
 	len = strlen( buf );
 	if ( debug ) {
 		fprintf( stderr, "got %d bytes\n", len );
 #ifdef LDAP_DEBUG
-		lber_bprint( buf, len );
+		ber_bprint( buf, len );
 #endif
 	}
 
@@ -431,22 +446,24 @@ do_queries( int s )
 		fprintf( fp, ".\r\n" );
 		rewind(fp);
 
-		exit( 0 );
+		exit( EXIT_SUCCESS );
 		/* NOT REACHED */
 	}
 
-	if ( (ld = ldap_open( ldaphost, ldapport )) == NULL ) {
-		if ( debug ) perror( "ldap_open" );
+	if ( (ld = ldap_init( ldaphost, ldapport )) == NULL ) {
+		if ( debug ) perror( "ldap_init" );
 		fprintf(fp, "0An error occurred (explanation)\tE%d\t%s\t%d\r\n",
 		    LDAP_SERVER_DOWN, myhost, myport );
 		fprintf( fp, ".\r\n" );
 		rewind(fp);
-		exit( 1 );
+		exit( EXIT_FAILURE );
 	}
 
-	ld->ld_deref = LDAP_DEREF_ALWAYS;
+	deref = LDAP_DEREF_ALWAYS;
 	if ( !searchaliases )
-		ld->ld_deref = LDAP_DEREF_FINDING;
+		deref = LDAP_DEREF_FINDING;
+
+	ldap_set_option(ld, LDAP_OPT_DEREF, &deref);
 
 	if ( (rc = ldap_simple_bind_s( ld, NULL, NULL ))
 	    != LDAP_SUCCESS ) {
@@ -455,7 +472,7 @@ do_queries( int s )
 		    rc, myhost, myport );
 		fprintf( fp, ".\r\n" );
 		rewind(fp);
-		exit( 1 );
+		exit( EXIT_FAILURE );
 	}
 
 	switch ( *query++ ) {
@@ -479,7 +496,7 @@ do_queries( int s )
 	fprintf( fp, ".\r\n" );
 	rewind(fp);
 
-	exit( 0 );
+	exit( EXIT_SUCCESS );
 	/* NOT REACHED */
 }
 
@@ -531,16 +548,16 @@ isnonleaf( LDAP *ld, char **oclist, char *dn )
 		struct timeval	timeout;
 		LDAPMessage	*res = NULL;
 		static char	*attrs[] = { "objectClass", 0 };
+		int sizelimit = 1;
 
 		timeout.tv_sec = GO500GW_TIMEOUT;
 		timeout.tv_usec = 0;
-
-		ld->ld_sizelimit = 1;
-
+		ldap_set_option(ld, LDAP_OPT_SIZELIMIT, &sizelimit);
 		if ( (rc = ldap_search_st( ld, dn, LDAP_SCOPE_ONELEVEL,
-		    "(objectClass=*)", attrs, 0, &timeout, &res ))
+		    NULL, attrs, 0, &timeout, &res ))
 		    == LDAP_SUCCESS || rc == LDAP_SIZELIMIT_EXCEEDED ) {
-			ld->ld_sizelimit = LDAP_NO_LIMIT;
+			sizelimit = LDAP_NO_LIMIT;
+			ldap_set_option(ld, LDAP_OPT_SIZELIMIT, &sizelimit);
 
 			numentries = ldap_count_entries( ld, res );
 			if ( res != NULL )
@@ -558,7 +575,7 @@ do_menu( LDAP *ld, FILE *fp, char *dn )
 {
 	char		**s;
 	char		*rdn = NULL;
-	FriendlyMap	*fm = NULL;
+	LDAPFriendlyMap	*fm = NULL;
 
 	if ( strcmp( dn, "" ) != 0 ) {
 		s = ldap_explode_dn( dn, 1 );
@@ -590,12 +607,14 @@ do_list( LDAP *ld, FILE *fp, char *dn )
 	int		rc;
 	LDAPMessage	*e, *res;
 	struct timeval	timeout;
-	FriendlyMap	*fm = NULL;
+	LDAPFriendlyMap	*fm = NULL;
 	static char	*attrs[] = { "objectClass", 0 };
+	int deref = LDAP_DEREF_FINDING;
 
 	timeout.tv_sec = GO500GW_TIMEOUT;
 	timeout.tv_usec = 0;
-	ld->ld_deref = LDAP_DEREF_FINDING;
+
+	ldap_set_option(ld, LDAP_OPT_DEREF, &deref);
 
 	if ( (rc = ldap_search_st( ld, dn, LDAP_SCOPE_ONELEVEL,
 	    "(!(objectClass=dSA))", attrs, 0, &timeout, &res )) != LDAP_SUCCESS
@@ -605,7 +624,8 @@ do_list( LDAP *ld, FILE *fp, char *dn )
 		return;
 	}
 
-	ld->ld_deref = LDAP_DEREF_ALWAYS;
+	deref = LDAP_DEREF_ALWAYS;
+	ldap_set_option(ld, LDAP_OPT_DEREF, &deref);
 
 	if ( ldap_count_entries( ld, res ) < 1 ) {
 		return;
@@ -684,7 +704,7 @@ make_scope( LDAP *ld, char *dn )
 
 	timeout.tv_sec = GO500GW_TIMEOUT;
 	timeout.tv_usec = 0;
-	if ( ldap_search_st( ld, dn, LDAP_SCOPE_BASE, "objectClass=*",
+	if ( ldap_search_st( ld, dn, LDAP_SCOPE_BASE, NULL,
 	    attrs, 0, &timeout, &res ) != LDAP_SUCCESS ) {
 		return( -1 );
 	}
@@ -705,6 +725,7 @@ make_scope( LDAP *ld, char *dn )
 static void
 do_search( LDAP *ld, FILE *fp, char *query )
 {
+	int deref;
 	int		scope;
 	char		*base, *filter;
 	char		*filtertype;
@@ -717,47 +738,28 @@ do_search( LDAP *ld, FILE *fp, char *query )
 
 	if ( (filter = strchr( query, '\t' )) == NULL ) {
 		fprintf( fp, "3Missing filter!\r\n" );
-		exit( 1 );
+		exit( EXIT_FAILURE );
 	}
 	*filter++ = '\0';
 	base = query;
 
-#ifdef GO500GW_UFN
-	if ( strchr( filter, ',' ) != NULL ) {
-		ldap_ufn_setprefix( ld, base );
-		timeout.tv_sec = GO500GW_TIMEOUT;
-		timeout.tv_usec = 0;
-		ldap_ufn_timeout( (void *) &timeout );
-
-		ld->ld_deref = LDAP_DEREF_FINDING;
-
-		if ( (rc = ldap_ufn_search_s( ld, filter, attrs, 0, &res ))
-		    != LDAP_SUCCESS && rc != LDAP_SIZELIMIT_EXCEEDED ) {
-			fprintf(fp,
-			    "0An error occurred (explanation)\t@%d\t%s\t%d\r\n",
-			    rc, myhost, myport );
-			return;
-		}
-
-		count = ldap_count_entries( ld, res );
-	} else {
-#endif
 		if ( (scope = make_scope( ld, base )) == -1 ) {
 			fprintf( fp, "3Bad scope\r\n" );
-			exit( 1 );
+			exit( EXIT_FAILURE );
 		}
 
 		filtertype = (scope == LDAP_SCOPE_ONELEVEL ?
 		    "go500gw onelevel" : "go500gw subtree");
-		ld->ld_deref = (scope == LDAP_SCOPE_ONELEVEL ?
+		deref = (scope == LDAP_SCOPE_ONELEVEL ?
 		    LDAP_DEREF_FINDING : LDAP_DEREF_ALWAYS);
+		ldap_set_option(ld, LDAP_OPT_DEREF, &deref);
 		timeout.tv_sec = GO500GW_TIMEOUT;
 		timeout.tv_usec = 0;
 
 		if ( (filtd = ldap_init_getfilter( filterfile )) == NULL ) {
 			fprintf( stderr, "Cannot open filter file (%s)\n",
 			    filterfile );
-			exit( 1 );
+			exit( EXIT_FAILURE );
 		}
 
 		count = 0;
@@ -775,19 +777,16 @@ do_search( LDAP *ld, FILE *fp, char *query )
 			if ( (count = ldap_count_entries( ld, res )) != 0 )
 				break;
 		}
-		ld->ld_deref = LDAP_DEREF_ALWAYS;
+		deref = LDAP_DEREF_ALWAYS;
+		ldap_set_option(ld, LDAP_OPT_DEREF, &deref);
 		ldap_getfilter_free( filtd );
-#ifdef GO500GW_UFN
-	}
-#endif
 
 	if ( count == 0 ) {
 		return;
 	}
 
 	if ( count == 1 ) {
-		char	*dn, **s, **oc;
-		int	rc;
+		char	*dn, **oc;
 
 		e = ldap_first_entry( ld, res );
 		oc = ldap_get_values( ld, e, "objectClass" );
@@ -837,7 +836,7 @@ do_search( LDAP *ld, FILE *fp, char *query )
 
 
 static int
-entry2textwrite( void *fp, char *buf, int len )
+entry2textwrite( void *fp, char *buf, ber_len_t len )
 {
         return( fwrite( buf, len, 1, (FILE *)fp ) == 0 ? -1 : len );
 }
@@ -852,10 +851,12 @@ do_read( LDAP *ld, FILE *fp, char *dn )
 	if ( ldap_entry2text_search( ld, dn, NULL, NULL, tmpllist, NULL, NULL,
 	    entry2textwrite,(void *) fp, "\r\n", rdncount, 0 )
 	    != LDAP_SUCCESS ) {
+		int ld_errno = 0;
+		ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ld_errno);
 
 		fprintf(fp,
-		    "0An error occurred (explanation)\t@%d\t%s\t%d\r\n",
-		    ld->ld_errno, myhost, myport );
+		    "0An error occurred (explanation)\t@%s\t%s\t%d\r\n",
+		    ldap_err2string( ld_errno ), myhost, myport );
 	}
 
 	if ( tmpllist != NULL ) {
