@@ -1,3 +1,4 @@
+/* $OpenLDAP: pkg/ldap/clients/mail500/main.c,v 1.25.2.5 2002/01/04 20:38:07 kurt Exp $ */
 /*
  * Copyright (c) 1990 Regents of the University of Michigan.
  * All rights reserved.
@@ -8,36 +9,37 @@
  * may not be used to endorse or promote products derived from this
  * software without specific prior written permission. This software
  * is provided ``as is'' without express or implied warranty.
+ *
+ * Copyright 1998-2002 The OpenLDAP Foundation
+ * COPYING RESTRICTIONS APPLY.  See COPYRIGHT File in top level directory
+ * of this package for details.
  */
 
 #include "portable.h"
 
 #include <stdio.h>
-#include <stdlib.h>
+
+#include <ac/stdlib.h>
 
 #include <ac/ctype.h>
+#include <ac/param.h>
 #include <ac/signal.h>
 #include <ac/string.h>
+#include <ac/sysexits.h>
 #include <ac/syslog.h>
 #include <ac/time.h>
-#include <ac/wait.h>
 #include <ac/unistd.h>
+#include <ac/wait.h>
 
 #include <sys/stat.h>
 
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
-
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
 
-#include <sysexits.h>
+#include <ldap.h>
 
-#include "lber.h"
-#include "ldap.h"
-#include "ldapconfig.h"
+#include "ldap_defaults.h"
 
 #ifndef MAIL500_BOUNCEFROM
 #define MAIL500_BOUNCEFROM "<>"
@@ -64,7 +66,7 @@ char	*mailfrom = NULL;
 char	*host = NULL;
 char	*ldaphost = NULL;
 int	hostlen = 0;
-int	debug = 0;
+int	debug;
 
 typedef struct errs {
 	int		e_code;
@@ -81,7 +83,7 @@ typedef struct errs {
 #define	E_NOOWNER		11
 #define E_GROUPUNKNOWN		12
 	char		*e_addr;
-	union {
+	union e_union_u {
 		char		*e_u_loop;
 		LDAPMessage	*e_u_msg;
 	} e_union;
@@ -105,20 +107,20 @@ typedef struct baseinfo {
 				/* (up to three of them) */
 } Base;
 
-Base	base[] = 
-	{ "ou=People, o=University of Michigan, c=US",
+Base	base[] = {
+	{"ou=People, dc=example, dc=com",
 		0, USER,
-		"uid=%s", "cn=%s", NULL,
-	  "ou=System Groups, ou=Groups, o=University of Michigan, c=US",
+		{"uid=%s", "cn=%s", NULL}},
+	{"ou=System Groups, ou=Groups, dc=example, dc=com",
 		1, 0xff,
-		"(&(cn=%s)(associatedDomain=%h))", NULL, NULL,
-	  "ou=User Groups, ou=Groups, o=University of Michigan, c=US",
+		{"(&(cn=%s)(associatedDomain=%h))", NULL, NULL}},
+	{"ou=User Groups, ou=Groups, dc=example, dc=com",
 		1, 0xff,
-		"(&(cn=%s)(associatedDomain=%h))", NULL, NULL,
-	  NULL
-	};
+		{"(&(cn=%s)(associatedDomain=%h))", NULL, NULL}},
+	{NULL}
+};
 
-char	*sendmailargs[] = { MAIL500_SENDMAIL, "-oMrX.500", "-odi", "-oi", "-f", NULL, NULL };
+char	*sendmailargs[] = { MAIL500_SENDMAIL, "-oMrLDAP", "-odi", "-oi", "-f", NULL, NULL };
 
 static char	*attrs[] = { "objectClass", "title", "postaladdress",
 			"telephoneNumber", "mail", "description", "owner",
@@ -172,14 +174,14 @@ main ( int argc, char **argv )
 
 #ifdef LOG_MAIL
 	openlog( myname, OPENLOG_OPTIONS, LOG_MAIL );
-#else
+#elif LOG_DEBUG
 	openlog( myname, OPENLOG_OPTIONS );
 #endif
 
 	while ( (i = getopt( argc, argv, "d:f:h:l:m:v:" )) != EOF ) {
 		switch( i ) {
 		case 'd':	/* turn on debugging */
-			debug |= atoi( optarg );
+			debug = atoi( optarg );
 			break;
 
 		case 'f':	/* who it's from & where errors should go */
@@ -357,12 +359,17 @@ main ( int argc, char **argv )
 static int
 connect_to_x500( void )
 {
-	if ( (ld = ldap_open( ldaphost, 0 )) == NULL ) {
-		syslog( LOG_ALERT, "ldap_open failed" );
+	int opt;
+
+	if ( (ld = ldap_init( ldaphost, 0 )) == NULL ) {
+		syslog( LOG_ALERT, "ldap_init failed" );
 		return( -1 );
 	}
-	ld->ld_sizelimit = MAIL500_MAXAMBIGUOUS;
-	ld->ld_deref = LDAP_DEREF_ALWAYS;
+
+	opt = MAIL500_MAXAMBIGUOUS;
+	ldap_set_option(ld, LDAP_OPT_SIZELIMIT, &opt);
+	opt = LDAP_DEREF_ALWAYS;
+	ldap_set_option(ld, LDAP_OPT_DEREF, &opt);
 
 	if ( ldap_simple_bind_s( ld, NULL, NULL ) != LDAP_SUCCESS ) {
 		syslog( LOG_ALERT, "ldap_simple_bind_s failed" );
@@ -509,9 +516,9 @@ do_address(
 	/* no matches - bounce with user unknown */
 	if ( match == 0 ) {
 		if ( type == USER ) {
-			add_error( err, nerr, E_USERUNKNOWN, name, NULLMSG );
+			add_error( err, nerr, E_USERUNKNOWN, name, NULL );
 		} else {
-			add_error( err, nerr, E_GROUPUNKNOWN, name, NULLMSG );
+			add_error( err, nerr, E_GROUPUNKNOWN, name, NULL );
 		}
 		return;
 	}
@@ -786,6 +793,7 @@ do_group_members(
 	char		filter[1024];
 	LDAPMessage	*ee, *res;
 	struct timeval	timeout;
+	int		opt;
 
 	/*
 	 * if all has gone according to plan, we've already arranged for
@@ -830,7 +838,7 @@ do_group_members(
 		if ( strcasecmp( joinable[0], "FALSE" ) == 0 ) {
 			if ( ! anymembers ) {
 				add_error( err, nerr, E_NOMEMBERS, dn,
-				    NULLMSG );
+				    NULL );
 			}
 
 			ldap_value_free( joinable );
@@ -844,7 +852,8 @@ do_group_members(
 		timeout.tv_usec = 0;
 
 		/* for each subtree to look in... */
-		ld->ld_sizelimit = MAIL500_MAXGROUPMEMBERS;
+		opt = MAIL500_MAXAMBIGUOUS;
+		ldap_set_option(ld, LDAP_OPT_SIZELIMIT, &opt);
 		for ( i = 0; base[i].b_dn != NULL; i++ ) {
 			/* find entries that have joined this group... */
 			rc = ldap_search_st( ld, base[i].b_dn,
@@ -896,7 +905,7 @@ do_group_members(
 					ndn = ldap_get_dn( ld, ee );
 
 					add_error( err, nerr,
-					    E_JOINMEMBERNOEMAIL, ndn, NULLMSG );
+					    E_JOINMEMBERNOEMAIL, ndn, NULL );
 
 					free( ndn );
 				}
@@ -904,11 +913,12 @@ do_group_members(
 
 			ldap_msgfree( res );
 		}
-		ld->ld_sizelimit = MAIL500_MAXAMBIGUOUS;
+		opt = MAIL500_MAXAMBIGUOUS;
+		ldap_set_option(ld, LDAP_OPT_SIZELIMIT, &opt);
 	}
 
 	if ( ! anymembers ) {
-		add_error( err, nerr, E_NOMEMBERS, dn, NULLMSG );
+		add_error( err, nerr, E_NOMEMBERS, dn, NULL );
 	}
 }
 
@@ -927,7 +937,7 @@ add_member(
 {
 	char		*ndn;
 	char		**mail;
-	int		i, rc;
+	int		rc;
 	LDAPMessage	*res, *e;
 	struct timeval	timeout;
 
@@ -936,7 +946,7 @@ add_member(
 	if ( (rc = ldap_search_st( ld, dn, LDAP_SCOPE_BASE, "(objectclass=*)",
 	    attrs, 0, &timeout, &res )) != LDAP_SUCCESS ) {
 		if ( rc == LDAP_NO_SUCH_OBJECT ) {
-			add_error( err, nerr, E_BADMEMBER, dn, NULLMSG );
+			add_error( err, nerr, E_BADMEMBER, dn, NULL );
 
 			return;
 		} else {
@@ -975,7 +985,7 @@ add_member(
 	} else {
 		if ( suppress == NULL || strcasecmp( suppress[0], "FALSE" )
 		    == 0 ) {
-			add_error( err, nerr, E_MEMBERNOEMAIL, ndn, NULLMSG );
+			add_error( err, nerr, E_MEMBERNOEMAIL, ndn, NULL );
 		}
 	}
 
@@ -1000,7 +1010,7 @@ do_group_request(
 
 		ldap_value_free( requeststo );
 	} else {
-		add_error( err, nerr, E_NOREQUEST, dn, NULLMSG );
+		add_error( err, nerr, E_NOREQUEST, dn, NULL );
 	}
 }
 
@@ -1022,7 +1032,7 @@ do_group_errors(
 
 		ldap_value_free( errorsto );
 	} else {
-		add_error( err, nerr, E_NOERRORS, dn, NULLMSG );
+		add_error( err, nerr, E_NOERRORS, dn, NULL );
 	}
 }
 
@@ -1042,7 +1052,7 @@ do_group_owner(
 		add_to( to, nto, owner );
 		ldap_value_free( owner );
 	} else {
-		add_error( err, nerr, E_NOOWNER, dn, NULLMSG );
+		add_error( err, nerr, E_NOOWNER, dn, NULL );
 	}
 }
 
@@ -1613,7 +1623,7 @@ get_attributes_mail_dn(
 )
 {
 	LDAPMessage	*ee, *res;
-	char		**vals, **dnlist, **mail, **grname, **graddr;
+	char		**vals, **dnlist, **mail, **grname;
         char            *dn;
 	int		nto = 0, i, rc;
 	struct timeval	timeout;
