@@ -1,3 +1,4 @@
+/* $OpenLDAP: pkg/ldap/clients/rcpt500/rcpt500.h,v 1.3.8.3 2000/08/18 02:32:52 kurt Exp $ */
 /*
  * rcpt500.h: includes for rcpt500 (X.500 email query responder)
  *
@@ -43,9 +44,6 @@ int	query_cmd LDAP_P((struct msginfo *msgp, char *reply));
 extern struct command rcpt_cmds[];
 /* main.c */
 extern int dosyslog;
-#ifdef LDAP_CONNECTIONLESS
-extern int do_cldap;
-#endif
 extern int derefaliases;
 extern int sizelimit;
 extern int rdncount;
