@@ -1,3 +1,8 @@
+/* $OpenLDAP: pkg/ldap/clients/ud/mod.c,v 1.10.8.6 2002/01/04 20:38:10 kurt Exp $ */
+/*
+ * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 /*
  * Copyright (c) 1991,1993  Regents of the University of Michigan.
  * All rights reserved.
@@ -13,13 +18,13 @@
 #include "portable.h"
 
 #include <stdio.h>
-#include <stdlib.h>
+
+#include <ac/stdlib.h>
 
 #include <ac/ctype.h>
 #include <ac/string.h>
 #include <ac/time.h>
 
-#include <lber.h>
 #include <ldap.h>
 #include "ud.h"
 
@@ -39,6 +44,7 @@ modify( char *who )
 #ifdef UOFM
 	static char printed_warning = 0;	/* for use with the */
 	struct attribute no_batch_update_attr;
+	int ld_errno;
 #endif
 	int is_a_group;		/* TRUE if it is; FALSE otherwise */
 
@@ -96,7 +102,10 @@ modify( char *who )
 	 */
 	no_batch_update_attr.quipu_name = "noBatchUpdates";
 	(void) fetch_boolean_value(dn, no_batch_update_attr);
-	if (verbose && !printed_warning && (ld->ld_errno == LDAP_NO_SUCH_ATTRIBUTE)) {
+
+	ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ld_errno);
+
+	if (verbose && !printed_warning && (ld_errno == LDAP_NO_SUCH_ATTRIBUTE)) {
 		printed_warning = 1;
 		printf("\n  WARNING!\n");
 		printf("  You are about to make a modification to an LDAP entry\n");
@@ -143,7 +152,7 @@ modify( char *who )
 		parse_answer(mp);
 		(void) ldap_msgfree(mp);
 	}
-	(void) Free(dn);
+	ldap_memfree(dn);
 	ldap_value_free(rdns);
 	return;
 }
@@ -370,7 +379,7 @@ get_value( char *id, char *prompt )
 	int count;		/* line # of new value -- if multiline */
 	int multiline = 0;	/* 1 if this value is multiline */
 	static char line[LINE_SIZE];	/* raw line from user */
-	static char buffer[MAX_DESC_LINES * LINE_SIZE];	/* holds ALL of the 
+	static char buffer[MAX_DESC_LINES * (LINE_SIZE+2)]; /* holds ALL of the
 							   lines we get */
 #ifdef DEBUG
 	if (debug & D_TRACE)
@@ -387,7 +396,7 @@ get_value( char *id, char *prompt )
 	 *  Other attributes may not.
 	 */
 	count = 1;
-	(void) memset(buffer, 0, sizeof(buffer));
+	(void) memset(buffer, '\0', sizeof(buffer));
 #ifdef UOFM
 	if (!strcmp(id, "postalAddress") || !strcmp(id, "homePostalAddress") || !strcmp(id, "multiLineDescription") || !strcmp(id, "vacationMessage")) 
 #else
@@ -457,9 +466,8 @@ mail_is_good:
 		 *  If the attribute which we are gathering is a "owner"
 		 *  then we should lookup the name.  The user is going to
 		 *  either have to change the search base before doing the
-		 *  modify, or the person is going to have to be within the
-		 *  scope of the current search base, or they will need to
-		 *  type in a UFN.
+		 *  modify or the person is going to have to be within the
+		 *  scope of the current search base.
 		 */
 		if (!strcmp(id, "owner")) {
 			LDAPMessage *lmp, *elmp;
@@ -479,7 +487,7 @@ mail_is_good:
 			}
 			tmp = ldap_get_dn(ld, elmp);
 			strcpy(buffer, tmp);
-			Free(tmp);
+			ldap_memfree(tmp);
 			(void) ldap_msgfree(lmp);
 			break;
 		}
@@ -500,7 +508,7 @@ mail_is_good:
 		 *  are done.
 		 */
 		if (count++ > 1)
-			(void) strcat(buffer, "$");
+			(void) strcat(buffer, " $ ");
 		(void) strcat(buffer, line);
 		if (!multiline)
 			break;
@@ -778,16 +786,22 @@ check_URL( char *url )
 void
 mod_perror( LDAP *ld )
 {
-	if ( ld == NULL || ( ld->ld_errno != LDAP_UNAVAILABLE &&
-	    ld->ld_errno != LDAP_UNWILLING_TO_PERFORM )) {
-		ldap_perror( ld, "modify" );
-		return;
+	int ld_errno = LDAP_SUCCESS;
+	char *ld_errtext = NULL;
+
+	ldap_get_option(ld, LDAP_OPT_ERROR_NUMBER, &ld_errno );
+
+	if( ld_errno != LDAP_SUCCESS ) {
+		ldap_get_option(ld, LDAP_OPT_ERROR_STRING, &ld_errtext );
+	}	
+
+	fprintf( stderr, "  modify failed: %s (%d)\n",
+		ldap_err2string( ld_errno ), ld_errno );
+
+	if( ld_errtext != NULL ) {
+		fprintf( stderr, "    additional information: %s\n",
+			ld_errtext );
 	}
 
-	fprintf( stderr, "\n  modify: failed because part of the online directory is not able\n" );
-	fprintf( stderr, "  to be modified right now" );
-	if ( ld->ld_errno == LDAP_UNAVAILABLE ) {
-		fprintf( stderr, " or is temporarily unavailable" );
-	}
-	fprintf( stderr, ".\n  Please try again later.\n" );
+	fprintf( stderr, "  Please try again later.\n" );
 }
