@@ -1,11 +1,13 @@
+/* $OpenLDAP: pkg/ldap/include/lutil_md5.h,v 1.10.8.5 2002/01/04 20:38:16 kurt Exp $ */
 /*
- * Copyright 1998,1999 The OpenLDAP Foundation, Redwood City, California, USA
+ * Copyright 1998-2002 The OpenLDAP Foundation, Redwood City, California, USA
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted only
- * as authorized by the OpenLDAP Public License.  A copy of this
- * license is available at http://www.OpenLDAP.org/license.html or
- * in file LICENSE in the top-level directory of the distribution.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.  A copy of this license is available at
+ * http://www.OpenLDAP.org/license.html or in file LICENSE in the
+ * top-level directory of the distribution.
  */
 
 /* See md5.c for explanation and copyright information.  */
@@ -13,42 +15,42 @@
 #ifndef _LUTIL_MD5_H_
 #define _LUTIL_MD5_H_
 
-#include <ldap_cdefs.h>
-#include <ac/bytes.h>
+#include <lber_types.h>
 
 LDAP_BEGIN_DECL
 
-/* Unlike previous versions of this code, uint32 need not be exactly
+/* Unlike previous versions of this code, ber_int_t need not be exactly
    32 bits, merely 32 bits or more.  Choosing a data type which is 32
    bits instead of 64 is not important; speed is considerably more
    important.  ANSI guarantees that "unsigned long" will be big enough,
    and always using it seems to have few disadvantages.  */
 
-#ifndef LDAP_UINT32
-#define LDAP_UINT32 1
-typedef LDAP_UINT4 uint32;
-#endif
+#define LUTIL_MD5_BYTES 16
 
 struct lutil_MD5Context {
-	uint32 buf[4];
-	uint32 bits[2];
+	ber_uint_t buf[4];
+	ber_uint_t bits[2];
 	unsigned char in[64];
 };
 
-LDAP_F void lutil_MD5Init LDAP_P((
+LDAP_LUTIL_F( void )
+lutil_MD5Init LDAP_P((
 	struct lutil_MD5Context *context));
 
-LDAP_F void lutil_MD5Update LDAP_P((
+LDAP_LUTIL_F( void )
+lutil_MD5Update LDAP_P((
 	struct lutil_MD5Context *context,
 	unsigned char const *buf,
-	unsigned len));
+	ber_len_t len));
 
-LDAP_F void lutil_MD5Final LDAP_P((
+LDAP_LUTIL_F( void )
+lutil_MD5Final LDAP_P((
 	unsigned char digest[16],
 	struct lutil_MD5Context *context));
 
-LDAP_F void lutil_MD5Transform LDAP_P((
-	uint32 buf[4],
+LDAP_LUTIL_F( void )
+lutil_MD5Transform LDAP_P((
+	ber_uint_t buf[4],
 	const unsigned char in[64]));
 
 /*
