@@ -1,5 +1,6 @@
+/* $OpenLDAP: pkg/ldap/libraries/libldap/free.c,v 1.12.6.4 2002/01/04 20:38:20 kurt Exp $ */
 /*
- * Copyright 1998-1999 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -13,14 +14,51 @@
 #include "portable.h"
 
 #include <stdio.h>
-#include <stdlib.h>
+#include <ac/stdlib.h>
 
-#include <ac/ctype.h>
 #include <ac/string.h>
 #include <ac/time.h>
 
-#include "lber.h"
-#include "ldap.h"
+#include "ldap-int.h"
+
+/*
+ * C-API deallocator
+ */
+void
+ldap_memfree( void *p )
+{
+	LDAP_FREE( p );
+}
+
+void
+ldap_memvfree( void **v )
+{
+	LDAP_VFREE( v );
+}
+
+void *
+ldap_memalloc( ber_len_t s )
+{
+	return LDAP_MALLOC( s );
+}
+
+void *
+ldap_memcalloc( ber_len_t n, ber_len_t s )
+{
+	return LDAP_CALLOC( n, s );
+}
+
+void *
+ldap_memrealloc( void* p, ber_len_t s )
+{
+	return LDAP_REALLOC( p, s );
+}
+
+char *
+ldap_strdup( LDAP_CONST char *p )
+{
+	return LDAP_STRDUP( p );
+}
 
 void
 ldap_getfilter_free( LDAPFiltDesc *lfdp )
@@ -31,31 +69,31 @@ ldap_getfilter_free( LDAPFiltDesc *lfdp )
     for ( flp = lfdp->lfd_filtlist; flp != NULL; flp = nextflp ) {
 	for ( fip = flp->lfl_ilist; fip != NULL; fip = nextfip ) {
 	    nextfip = fip->lfi_next;
-	    free( fip->lfi_filter );
-	    free( fip->lfi_desc );
-	    free( fip );
+	    LDAP_FREE( fip->lfi_filter );
+	    LDAP_FREE( fip->lfi_desc );
+	    LDAP_FREE( fip );
 	}
 	nextflp = flp->lfl_next;
-	free( flp->lfl_pattern );
-	free( flp->lfl_delims );
-	free( flp->lfl_tag );
-	free( flp );
+	LDAP_FREE( flp->lfl_pattern );
+	LDAP_FREE( flp->lfl_delims );
+	LDAP_FREE( flp->lfl_tag );
+	LDAP_FREE( flp );
     }
 
     if ( lfdp->lfd_curvalcopy != NULL ) {
-	free( lfdp->lfd_curvalcopy );
+	LDAP_FREE( lfdp->lfd_curvalcopy );
     }
     if ( lfdp->lfd_curvalwords != NULL ) {
-	free( lfdp->lfd_curvalwords );
+	LDAP_FREE( lfdp->lfd_curvalwords );
     }
     if ( lfdp->lfd_filtprefix != NULL ) {
-	free( lfdp->lfd_filtprefix );
+	LDAP_FREE( lfdp->lfd_filtprefix );
     }
     if ( lfdp->lfd_filtsuffix != NULL ) {
-	free( lfdp->lfd_filtsuffix );
+	LDAP_FREE( lfdp->lfd_filtsuffix );
     }
 
-    free( lfdp );
+    LDAP_FREE( lfdp );
 }
 
 /*
@@ -74,13 +112,21 @@ ldap_mods_free( LDAPMod **mods, int freemods )
 
 	for ( i = 0; mods[i] != NULL; i++ ) {
 		if ( mods[i]->mod_op & LDAP_MOD_BVALUES ) {
-			ber_bvecfree( mods[i]->mod_bvalues );
-		} else {
-			ldap_value_free( mods[i]->mod_values );
+			if( mods[i]->mod_bvalues != NULL )
+				ber_bvecfree( mods[i]->mod_bvalues );
+
+		} else if( mods[i]->mod_values != NULL ) {
+			LDAP_VFREE( mods[i]->mod_values );
 		}
-		free( (char *) mods[i] );
+
+		if ( mods[i]->mod_type != NULL ) {
+			LDAP_FREE( mods[i]->mod_type );
+		}
+
+		LDAP_FREE( (char *) mods[i] );
 	}
 
-	if ( freemods )
-		free( (char *) mods );
+	if ( freemods ) {
+		LDAP_FREE( (char *) mods );
+	}
 }
