@@ -1,5 +1,6 @@
+/* $OpenLDAP: pkg/ldap/libraries/libldap/getvalues.c,v 1.12.4.3 2002/01/04 20:38:20 kurt Exp $ */
 /*
- * Copyright 1998-1999 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -12,32 +13,35 @@
 #include "portable.h"
 
 #include <stdio.h>
-#include <stdlib.h>
+
+#include <ac/stdlib.h>
 
 #include <ac/ctype.h>
 #include <ac/socket.h>
 #include <ac/string.h>
 #include <ac/time.h>
 
-#include "lber.h"
-#include "ldap.h"
+#include "ldap-int.h"
 
 char **
-ldap_get_values( LDAP *ld, LDAPMessage *entry, char *target )
+ldap_get_values( LDAP *ld, LDAPMessage *entry, LDAP_CONST char *target )
 {
 	BerElement	ber;
-	char		attr[LDAP_MAX_ATTR_LEN];
+	char		*attr;
 	int		found = 0;
-	long		len;
 	char		**vals;
 
+	assert( ld != NULL );
+	assert( LDAP_VALID( ld ) );
+	assert( entry != NULL );
+	assert( target != NULL );
+
 	Debug( LDAP_DEBUG_TRACE, "ldap_get_values\n", 0, 0, 0 );
 
 	ber = *entry->lm_ber;
 
 	/* skip sequence, dn, sequence of, and snag the first attr */
-	len = sizeof(attr);
-	if ( ber_scanf( &ber, "{x{{s", attr, &len ) == LBER_ERROR ) {
+	if ( ber_scanf( &ber, "{x{{a" /*}}}*/, &attr ) == LBER_ERROR ) {
 		ld->ld_errno = LDAP_DECODING_ERROR;
 		return( NULL );
 	}
@@ -47,16 +51,22 @@ ldap_get_values( LDAP *ld, LDAPMessage *entry, char *target )
 
 	/* break out on success, return out on error */
 	while ( ! found ) {
-		len = sizeof(attr);
-		if ( ber_scanf( &ber, "x}{s", attr, &len ) == LBER_ERROR ) {
+		LDAP_FREE(attr);
+		attr = NULL;
+
+		if ( ber_scanf( &ber, /*{*/ "x}{a" /*}*/, &attr ) == LBER_ERROR ) {
 			ld->ld_errno = LDAP_DECODING_ERROR;
 			return( NULL );
 		}
 
 		if ( strcasecmp( target, attr ) == 0 )
 			break;
+
 	}
 
+	LDAP_FREE(attr);
+	attr = NULL;
+
 	/* 
 	 * if we get this far, we've found the attribute and are sitting
 	 * just before the set of values.
@@ -71,21 +81,24 @@ ldap_get_values( LDAP *ld, LDAPMessage *entry, char *target )
 }
 
 struct berval **
-ldap_get_values_len( LDAP *ld, LDAPMessage *entry, char *target )
+ldap_get_values_len( LDAP *ld, LDAPMessage *entry, LDAP_CONST char *target )
 {
 	BerElement	ber;
-	char		attr[LDAP_MAX_ATTR_LEN];
+	char		*attr;
 	int		found = 0;
-	long		len;
 	struct berval	**vals;
 
+	assert( ld != NULL );
+	assert( LDAP_VALID( ld ) );
+	assert( entry != NULL );
+	assert( target != NULL );
+
 	Debug( LDAP_DEBUG_TRACE, "ldap_get_values_len\n", 0, 0, 0 );
 
 	ber = *entry->lm_ber;
 
 	/* skip sequence, dn, sequence of, and snag the first attr */
-	len = sizeof(attr);
-	if ( ber_scanf( &ber, "{x{{s", attr, &len ) == LBER_ERROR ) {
+	if ( ber_scanf( &ber, "{x{{a" /* }}} */, &attr ) == LBER_ERROR ) {
 		ld->ld_errno = LDAP_DECODING_ERROR;
 		return( NULL );
 	}
@@ -95,8 +108,10 @@ ldap_get_values_len( LDAP *ld, LDAPMessage *entry, char *target )
 
 	/* break out on success, return out on error */
 	while ( ! found ) {
-		len = sizeof(attr);
-		if ( ber_scanf( &ber, "x}{s", attr, &len ) == LBER_ERROR ) {
+		LDAP_FREE( attr );
+		attr = NULL;
+
+		if ( ber_scanf( &ber, /*{*/ "x}{a" /*}*/, &attr ) == LBER_ERROR ) {
 			ld->ld_errno = LDAP_DECODING_ERROR;
 			return( NULL );
 		}
@@ -105,6 +120,9 @@ ldap_get_values_len( LDAP *ld, LDAPMessage *entry, char *target )
 			break;
 	}
 
+	LDAP_FREE( attr );
+	attr = NULL;
+
 	/* 
 	 * if we get this far, we've found the attribute and are sitting
 	 * just before the set of values.
@@ -141,25 +159,11 @@ ldap_count_values_len( struct berval **vals )
 void
 ldap_value_free( char **vals )
 {
-	int	i;
-
-	if ( vals == NULL )
-		return;
-	for ( i = 0; vals[i] != NULL; i++ )
-		free( vals[i] );
-	free( (char *) vals );
+	LDAP_VFREE( vals );
 }
 
 void
 ldap_value_free_len( struct berval **vals )
 {
-	int	i;
-
-	if ( vals == NULL )
-		return;
-	for ( i = 0; vals[i] != NULL; i++ ) {
-		free( vals[i]->bv_val );
-		free( vals[i] );
-	}
-	free( (char *) vals );
+	ber_bvecfree( vals );
 }
