@@ -1,3 +1,8 @@
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/base64.c,v 1.6.2.4 2002/01/04 20:38:24 kurt Exp $ */
+/*
+ * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 /*
  * Modified by Kurt D. Zeilenga for inclusion into OpenLDAP
  */
@@ -46,10 +51,9 @@
 
 #include "portable.h"
 
-#include <stdlib.h>
-#include <stddef.h>
-#include <ctype.h>
-
+#include <ac/assert.h>
+#include <ac/stdlib.h>
+#include <ac/ctype.h>
 #include <ac/string.h>
 
 /* include socket.h to get sys/types.h and/or winsock2.h */
@@ -57,8 +61,6 @@
 
 #include "lutil.h"
 
-#define Assert(Cond) if (!(Cond)) abort()
-
 static const char Base64[] =
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 static const char Pad64 = '=';
@@ -148,10 +150,10 @@ lutil_b64_ntop(
 		output[1] = ((input[0] & 0x03) << 4) + (input[1] >> 4);
 		output[2] = ((input[1] & 0x0f) << 2) + (input[2] >> 6);
 		output[3] = input[2] & 0x3f;
-		Assert(output[0] < 64);
-		Assert(output[1] < 64);
-		Assert(output[2] < 64);
-		Assert(output[3] < 64);
+		assert(output[0] < 64);
+		assert(output[1] < 64);
+		assert(output[2] < 64);
+		assert(output[3] < 64);
 
 		if (datalength + 4 > targsize)
 			return (-1);
@@ -171,9 +173,9 @@ lutil_b64_ntop(
 		output[0] = input[0] >> 2;
 		output[1] = ((input[0] & 0x03) << 4) + (input[1] >> 4);
 		output[2] = ((input[1] & 0x0f) << 2) + (input[2] >> 6);
-		Assert(output[0] < 64);
-		Assert(output[1] < 64);
-		Assert(output[2] < 64);
+		assert(output[0] < 64);
+		assert(output[1] < 64);
+		assert(output[2] < 64);
 
 		if (datalength + 4 > targsize)
 			return (-1);
