@@ -1,4 +1,9 @@
 /* idl.c - ldap id list handling routines */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/idl.c,v 1.29.2.9 2002/01/04 20:38:34 kurt Exp $ */
+/*
+ * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 
 #include "portable.h"
 
@@ -7,15 +12,63 @@
 #include <ac/string.h>
 #include <ac/socket.h>
 
-#include "ldapconfig.h"
 #include "slap.h"
 #include "back-ldbm.h"
 
 static ID_BLOCK* idl_dup( ID_BLOCK *idl );
 
+static void cont_alloc( Datum *cont, Datum *key )
+{
+	ldbm_datum_init( *cont );
+	cont->dsize = 1 + sizeof(ID) + key->dsize;
+	cont->dptr = ch_malloc( cont->dsize );
+
+	* (unsigned char *) cont->dptr = SLAP_INDEX_CONT_PREFIX;
+
+	AC_MEMCPY( &((unsigned char *)cont->dptr)[1 + sizeof(ID)],
+		key->dptr, key->dsize );
+}
+
+static void cont_id( Datum *cont, ID id )
+{
+	int i;
+
+	for( i=1; i <= sizeof(id); i++) {
+		((unsigned char *)cont->dptr)[i] = (unsigned char)(id & 0xFF);
+		id >>= 8;
+	}
+
+}
+
+static void cont_free( Datum *cont )
+{
+	ch_free( cont->dptr );
+}
+
+#ifdef LDBM_DEBUG_IDL
+static void idl_check(ID_BLOCK *idl)
+{
+	int i;
+	ID_BLOCK last;
+
+	if( ID_BLOCK_INDIRECT(idl) || ID_BLOCK_ALLIDS(idl)
+		|| ID_BLOCK_NIDS(idl) <= 1 )
+	{
+		return;
+	}
+
+	for( last = ID_BLOCK_ID(idl, 0), i = 1;
+		i < ID_BLOCK_NIDS(idl);
+		last = ID_BLOCK_ID(idl, i), i++ )
+	{
+		assert (last < ID_BLOCK_ID(idl, i) );
+	}
+}
+#endif
+
 /* Allocate an ID_BLOCK with room for nids ids */
 ID_BLOCK *
-idl_alloc( int nids )
+idl_alloc( unsigned int nids )
 {
 	ID_BLOCK	*new;
 
@@ -33,15 +86,18 @@ ID_BLOCK	*
 idl_allids( Backend *be )
 {
 	ID_BLOCK	*idl;
+	ID		id;
 
 	idl = idl_alloc( 0 );
 	ID_BLOCK_NMAX(idl) = ID_BLOCK_ALLIDS_VALUE;
-	ID_BLOCK_NIDS(idl) = next_id_get( be );
+	if ( next_id_get( be, &id ) ) {
+		return NULL;
+	}
+	ID_BLOCK_NIDS(idl) = id;
 
 	return( idl );
 }
 
-
 /* Free an ID_BLOCK */
 void
 idl_free( ID_BLOCK *idl )
@@ -61,15 +117,13 @@ idl_free( ID_BLOCK *idl )
 static ID_BLOCK *
 idl_fetch_one(
     Backend		*be,
-    struct dbcache	*db,
+    DBCache	*db,
     Datum		key
 )
 {
 	Datum	data;
 	ID_BLOCK	*idl;
 
-	ldbm_datum_init( data );
-
 	/* Debug( LDAP_DEBUG_TRACE, "=> idl_fetch_one\n", 0, 0, 0 ); */
 
 	data = ldbm_cache_fetch( db, key );
@@ -78,10 +132,11 @@ idl_fetch_one(
 		return NULL;
 	}
 
-	idl = idl_dup( (ID_BLOCK *) data.dptr);
+	idl = idl_dup((ID_BLOCK *) data.dptr);
+
 	ldbm_datum_free( db->dbc_db, data );
 
-	return( idl );
+	return idl;
 }
 
 
@@ -97,14 +152,13 @@ idl_fetch_one(
 ID_BLOCK *
 idl_fetch(
     Backend		*be,
-    struct dbcache	*db,
+    DBCache	*db,
     Datum		key
 )
 {
 	Datum	data;
 	ID_BLOCK	*idl;
 	ID_BLOCK	**tmp;
-	char	*kstr;
 	int	i, nids;
 
 	idl = idl_fetch_one( be, db, key );
@@ -119,7 +173,7 @@ idl_fetch(
 		idl_free( idl );
 		idl = idl_allids( be );
 
-		return idl;
+		return( idl );
 	}
 
 	if ( ! ID_BLOCK_INDIRECT( idl ) ) {
@@ -139,25 +193,21 @@ idl_fetch(
 	tmp = (ID_BLOCK **) ch_malloc( (i + 1) * sizeof(ID_BLOCK *) );
 
 	/* read in all the blocks */
-	kstr = (char *) ch_malloc( key.dsize + 20 );
+	cont_alloc( &data, &key );
 	nids = 0;
 	for ( i = 0; !ID_BLOCK_NOID(idl, i); i++ ) {
-		ldbm_datum_init( data );
-
-		sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr, ID_BLOCK_ID(idl, i) );
-		data.dptr = kstr;
-		data.dsize = strlen( kstr ) + 1;
+		cont_id( &data, ID_BLOCK_ID(idl, i) );
 
 		if ( (tmp[i] = idl_fetch_one( be, db, data )) == NULL ) {
 			Debug( LDAP_DEBUG_ANY,
-			    "idl_fetch of (%s) returns NULL\n", data.dptr, 0, 0 );
+			    "idl_fetch: one returned NULL\n", 0, 0, 0 );
 			continue;
 		}
 
 		nids += ID_BLOCK_NIDS(tmp[i]);
 	}
 	tmp[i] = NULL;
-	free( kstr );
+	cont_free( &data );
 	idl_free( idl );
 
 	/* allocate space for the big block */
@@ -171,7 +221,7 @@ idl_fetch(
 			continue;
 		}
 
-		SAFEMEMCPY(
+		AC_MEMCPY(
 			(char *) &ID_BLOCK_ID(idl, nids),
 			(char *) &ID_BLOCK_ID(tmp[i], 0),
 			ID_BLOCK_NIDS(tmp[i]) * sizeof(ID) );
@@ -181,7 +231,11 @@ idl_fetch(
 	}
 	free( (char *) tmp );
 
-	Debug( LDAP_DEBUG_TRACE, "<= idl_fetch %lu ids (%lu max)\n",
+#ifdef LDBM_DEBUG_IDL
+	idl_check(idl);
+#endif
+
+	Debug( LDAP_DEBUG_TRACE, "<= idl_fetch %ld ids (%ld max)\n",
 	       ID_BLOCK_NIDS(idl), ID_BLOCK_NMAX(idl), 0 );
 	return( idl );
 }
@@ -191,7 +245,7 @@ idl_fetch(
 static int
 idl_store(
     Backend		*be,
-    struct dbcache	*db,
+    DBCache	*db,
     Datum		key, 
     ID_BLOCK		*idl
 )
@@ -200,6 +254,10 @@ idl_store(
 	Datum	data;
 	struct ldbminfo *li = (struct ldbminfo *) be->be_private;
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(idl);
+#endif
+
 	ldbm_datum_init( data );
 
 	/* Debug( LDAP_DEBUG_TRACE, "=> idl_store\n", 0, 0, 0 ); */
@@ -213,14 +271,12 @@ idl_store(
 #endif
 
 	flags = LDBM_REPLACE;
-	if( li->li_dbcachewsync ) flags |= LDBM_SYNC;
 	rc = ldbm_cache_store( db, key, data, flags );
 
 	/* Debug( LDAP_DEBUG_TRACE, "<= idl_store %d\n", rc, 0, 0 ); */
 	return( rc );
 }
 
-
 /* split the block at id 
  *	locate ID greater than or equal to id.
  */
@@ -252,7 +308,7 @@ idl_split_block(
 		ID_BLOCK_NIDS(*right) = 1;
 		ID_BLOCK_ID(*right, 0) = id;
 	} else {
-		SAFEMEMCPY(
+		AC_MEMCPY(
 			(char *) &ID_BLOCK_ID(*right, 0),
 			(char *) &ID_BLOCK_ID(b, 0),
 			nr * sizeof(ID) );
@@ -261,11 +317,16 @@ idl_split_block(
 	}
 
 	/* the id being inserted & everything after in the second block */
-	SAFEMEMCPY(
+	AC_MEMCPY(
 		(char *) &ID_BLOCK_ID(*left, (nr == 0 ? 0 : 1)),
 	    (char *) &ID_BLOCK_ID(b, nr),
 		nl * sizeof(ID) );
 	ID_BLOCK_NIDS(*left) = nl + (nr == 0 ? 0 : 1);
+
+#ifdef LDBM_DEBUG_IDL
+	idl_check(*right);
+	idl_check(*left);
+#endif
 }
 
 
@@ -277,7 +338,7 @@ idl_split_block(
 static int
 idl_change_first(
     Backend		*be,
-    struct dbcache	*db,
+    DBCache	*db,
     Datum		hkey,		/* header block key	*/
     ID_BLOCK		*h,		/* header block 	*/
     int			pos,		/* pos in h to update	*/
@@ -292,17 +353,17 @@ idl_change_first(
 	/* delete old key block */
 	if ( (rc = ldbm_cache_delete( db, bkey )) != 0 ) {
 		Debug( LDAP_DEBUG_ANY,
-		    "ldbm_delete of (%s) returns %d\n", bkey.dptr, rc,
-		    0 );
+		    "idl_change_first: ldbm_cache_delete returned %d\n",
+			rc, 0, 0 );
 		return( rc );
 	}
 
 	/* write block with new key */
-	sprintf( bkey.dptr, "%c%s%ld", CONT_PREFIX, hkey.dptr, ID_BLOCK_ID(b, 0) );
-	bkey.dsize = strlen( bkey.dptr ) + 1;
+	cont_id( &bkey, ID_BLOCK_ID(b, 0) );
+
 	if ( (rc = idl_store( be, db, bkey, b )) != 0 ) {
 		Debug( LDAP_DEBUG_ANY,
-		    "idl_store of (%s) returns %d\n", bkey.dptr, rc, 0 );
+		    "idl_change_first: idl_store returned %d\n", rc, 0, 0 );
 		return( rc );
 	}
 
@@ -310,7 +371,7 @@ idl_change_first(
 	ID_BLOCK_ID(h, pos) = ID_BLOCK_ID(b, 0);
 	if ( (rc = idl_store( be, db, hkey, h )) != 0 ) {
 		Debug( LDAP_DEBUG_ANY,
-		    "idl_store of (%s) returns %d\n", hkey.dptr, rc, 0 );
+		    "idl_change_first: idl_store returned %d\n", rc, 0, 0 );
 		return( rc );
 	}
 
@@ -321,24 +382,16 @@ idl_change_first(
 int
 idl_insert_key(
     Backend		*be,
-    struct dbcache	*db,
+    DBCache	*db,
     Datum		key,
     ID			id
 )
 {
 	int	i, j, first, rc;
 	ID_BLOCK	*idl, *tmp, *tmp2, *tmp3;
-	char	*kstr;
 	Datum	k2;
 
-	ldbm_datum_init( k2 );
-
 	if ( (idl = idl_fetch_one( be, db, key )) == NULL ) {
-#ifdef LDBM_DEBUG
-		Statslog( LDAP_DEBUG_STATS, "=> idl_insert_key(): no key yet\n",
-			0, 0, 0, 0, 0 );
-#endif
-
 		idl = idl_alloc( 1 );
 		ID_BLOCK_ID(idl, ID_BLOCK_NIDS(idl)++) = id;
 		rc = idl_store( be, db, key, idl );
@@ -388,22 +441,16 @@ idl_insert_key(
 			/* store it */
 			rc = idl_store( be, db, key, idl );
 
-			/* store the first id block */
-			kstr = (char *) ch_malloc( key.dsize + 20 );
-			sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr,
-			    ID_BLOCK_ID(tmp, 0) );
-			k2.dptr = kstr;
-			k2.dsize = strlen( kstr ) + 1;
+			cont_alloc( &k2, &key );
+			cont_id( &k2, ID_BLOCK_ID(tmp, 0) );
+
 			rc = idl_store( be, db, k2, tmp );
 
-			/* store the second id block */
-			sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr,
-			    ID_BLOCK_ID(tmp2, 0) );
-			k2.dptr = kstr;
-			k2.dsize = strlen( kstr ) + 1;
+			cont_id( &k2, ID_BLOCK_ID(tmp2, 0) );
 			rc = idl_store( be, db, k2, tmp2 );
 
-			free( kstr );
+			cont_free( &k2 );
+
 			idl_free( tmp );
 			idl_free( tmp2 );
 			break;
@@ -424,6 +471,7 @@ idl_insert_key(
 	/* select the block to try inserting into *//* XXX linear search XXX */
 	for ( i = 0; !ID_BLOCK_NOID(idl, i) && id > ID_BLOCK_ID(idl, i); i++ )
 		;	/* NULL */
+
 	if ( i != 0 ) {
 		i--;
 		first = 0;
@@ -432,14 +480,13 @@ idl_insert_key(
 	}
 
 	/* get the block */
-	kstr = (char *) ch_malloc( key.dsize + 20 );
-	sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr, ID_BLOCK_ID(idl, i) );
-	k2.dptr = kstr;
-	k2.dsize = strlen( kstr ) + 1;
+	cont_alloc( &k2, &key );
+	cont_id( &k2, ID_BLOCK_ID(idl, i) );
+
 	if ( (tmp = idl_fetch_one( be, db, k2 )) == NULL ) {
-		Debug( LDAP_DEBUG_ANY, "nonexistent continuation block (%s)\n",
-		    k2.dptr, 0, 0 );
-		free( kstr );
+		Debug( LDAP_DEBUG_ANY, "idl_insert_key: nonexistent continuation block\n",
+		    0, 0, 0 );
+		cont_free( &k2 );
 		idl_free( idl );
 		return( -1 );
 	}
@@ -449,7 +496,7 @@ idl_insert_key(
 	case 0:		/* id inserted ok */
 		if ( (rc = idl_store( be, db, k2, tmp )) != 0 ) {
 			Debug( LDAP_DEBUG_ANY,
-			    "idl_store of (%s) returns %d\n", k2.dptr, rc, 0 );
+			    "idl_insert_key: idl_store returned %d\n", rc, 0, 0 );
 		}
 		break;
 
@@ -478,16 +525,14 @@ idl_insert_key(
 		/* is there a next block? */
 		if ( !first && !ID_BLOCK_NOID(idl, i + 1) ) {
 			/* read it in */
-			k2.dptr = (char *) ch_malloc( key.dsize + 20 );
-			sprintf( k2.dptr, "%c%s%ld", CONT_PREFIX, key.dptr,
-			    ID_BLOCK_ID(idl, i + 1) );
-			k2.dsize = strlen( k2.dptr ) + 1;
+			cont_alloc( &k2, &key );
+			cont_id( &k2, ID_BLOCK_ID(idl, i) );
 			if ( (tmp2 = idl_fetch_one( be, db, k2 )) == NULL ) {
 				Debug( LDAP_DEBUG_ANY,
-				    "idl_fetch_one (%s) returns NULL\n",
-				    k2.dptr, 0, 0 );
+				    "idl_insert_key: idl_fetch_one returned NULL\n",
+				    0, 0, 0 );
 				/* split the original block */
-				free( k2.dptr );
+				cont_free( &k2 );
 				goto split;
 			}
 
@@ -507,15 +552,16 @@ idl_insert_key(
 			     * ID off the end of it.
 			     */
 			    rc = idl_insert( &tmp, id, db->dbc_maxids );
-			    assert( rc == 0 );
-			    k3.dptr = kstr;
-			    k3.dsize = strlen( kstr ) + 1;
+
+				k3.dptr = ch_malloc(k2.dsize);
+				k3.dsize = k2.dsize;
+				AC_MEMCPY(k3.dptr, k2.dptr, k3.dsize);
 			    if ( (rc = idl_store( be, db, k3, tmp )) != 0 ) {
 				Debug( LDAP_DEBUG_ANY,
-			    "idl_store of (%s) returns %d\n", k3.dptr, rc, 0 );
+			    "idl_insert_key: idl_store returned %d\n", rc, 0, 0 );
 			    }
-			    free( kstr );
-			    kstr = k2.dptr;
+
+				free( k3.dptr );
 
 			    id = id2;
 			    /* This new id will necessarily be inserted
@@ -532,13 +578,17 @@ idl_insert_key(
 				/* FALL */
 
 			case 2:		/* id already there - how? */
-			case 0:		/* id inserted */
+			case 0:		/* id inserted: this can never be
+					 * the result of idl_insert, because
+					 * we guaranteed that idl_change_first
+					 * will always be called.
+					 */
 				if ( rc == 2 ) {
 					Debug( LDAP_DEBUG_ANY,
-					    "id %lu already in next block\n",
+					    "idl_insert_key: id %ld already in next block\n",
 					    id, 0, 0 );
 				}
-				free( kstr );
+
 				idl_free( tmp );
 				idl_free( tmp2 );
 				idl_free( idl );
@@ -576,10 +626,7 @@ split:
 
 			/* delete all indirect blocks */
 			for ( j = 0; !ID_BLOCK_NOID(idl, j); j++ ) {
-				sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr,
-				    ID_BLOCK_ID(idl, j) );
-				k2.dptr = kstr;
-				k2.dsize = strlen( kstr ) + 1;
+				cont_id( &k2, ID_BLOCK_ID(idl, j) );
 
 				rc = ldbm_cache_delete( db, k2 );
 			}
@@ -589,7 +636,7 @@ split:
 			idl = idl_allids( be );
 			rc = idl_store( be, db, key, idl );
 
-			free( kstr );
+			cont_free( &k2 );
 			idl_free( idl );
 			idl_free( tmp );
 			return( rc );
@@ -602,7 +649,7 @@ split:
 		tmp = idl_alloc( ID_BLOCK_NMAX(idl) + 1 );
 		ID_BLOCK_NIDS(tmp) = ID_BLOCK_INDIRECT_VALUE;
 		/* everything up to the split block */
-		SAFEMEMCPY(
+		AC_MEMCPY(
 			(char *) &ID_BLOCK_ID(tmp, 0),
 			(char *) &ID_BLOCK_ID(idl, 0),
 		    i * sizeof(ID) );
@@ -610,7 +657,7 @@ split:
 		ID_BLOCK_ID(tmp, i) = ID_BLOCK_ID(tmp2, 0);
 		ID_BLOCK_ID(tmp, i + 1) = ID_BLOCK_ID(tmp3, 0);
 		/* everything after the split block */
-		SAFEMEMCPY(
+		AC_MEMCPY(
 			(char *) &ID_BLOCK_ID(tmp, i + 2),
 			(char *) &ID_BLOCK_ID(idl, i + 1),
 			(ID_BLOCK_NMAX(idl) - i - 1) * sizeof(ID) );
@@ -619,17 +666,11 @@ split:
 		rc = idl_store( be, db, key, tmp );
 
 		/* store the first id block */
-		sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr,
-		    ID_BLOCK_ID(tmp2, 0) );
-		k2.dptr = kstr;
-		k2.dsize = strlen( kstr ) + 1;
+		cont_id( &k2, ID_BLOCK_ID(tmp2, 0) );
 		rc = idl_store( be, db, k2, tmp2 );
 
 		/* store the second id block */
-		sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr,
-		    ID_BLOCK_ID(tmp3, 0) );
-		k2.dptr = kstr;
-		k2.dsize = strlen( kstr ) + 1;
+		cont_id( &k2, ID_BLOCK_ID(tmp3, 0) );
 		rc = idl_store( be, db, k2, tmp3 );
 
 		idl_free( tmp2 );
@@ -637,7 +678,7 @@ split:
 		break;
 	}
 
-	free( kstr );
+	cont_free( &k2 );
 	idl_free( tmp );
 	idl_free( idl );
 	return( rc );
@@ -654,9 +695,9 @@ split:
  *		3	id not inserted, block must be split
  */
 int
-idl_insert( ID_BLOCK **idl, ID id, int maxids )
+idl_insert( ID_BLOCK **idl, ID id, unsigned int maxids )
 {
-	unsigned int	i, j;
+	unsigned int	i;
 
 	if ( ID_BLOCK_ALLIDS( *idl ) ) {
 		return( 2 );	/* already there */
@@ -685,10 +726,10 @@ idl_insert( ID_BLOCK **idl, ID id, int maxids )
 		    (ID_BLOCK_NMAX(*idl) + ID_BLOCK_IDS_OFFSET) * sizeof(ID) );
 	}
 
-	/* make a slot for the new id *//* XXX bubble move XXX */
-	for ( j = ID_BLOCK_NIDS(*idl); j != i; j-- ) {
-		ID_BLOCK_ID(*idl, j) = ID_BLOCK_ID(*idl, j-1);
-	}
+	/* make a slot for the new id */
+	AC_MEMCPY( &ID_BLOCK_ID(*idl, i+1), &ID_BLOCK_ID(*idl, i),
+	            (ID_BLOCK_NIDS(*idl) - i) * sizeof(ID) );
+
 	ID_BLOCK_ID(*idl, i) = id;
 	ID_BLOCK_NIDS(*idl)++;
 	(void) memset(
@@ -696,6 +737,10 @@ idl_insert( ID_BLOCK **idl, ID id, int maxids )
 		'\0',
 	    (ID_BLOCK_NMAX(*idl) - ID_BLOCK_NIDS(*idl)) * sizeof(ID) );
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(*idl);
+#endif
+
 	return( i == 0 ? 1 : 0 );	/* inserted - first id changed or not */
 }
 
@@ -703,16 +748,15 @@ idl_insert( ID_BLOCK **idl, ID id, int maxids )
 int
 idl_delete_key (
 	Backend         *be,
-	struct dbcache  *db,
+	DBCache  *db,
 	Datum           key,
 	ID              id
 )
 {
 	Datum  data;
-	ID_BLOCK *idl, *tmp;
+	ID_BLOCK *idl;
 	unsigned i;
 	int j, nids;
-	char	*kstr;
 
 	if ( (idl = idl_fetch_one( be, db, key ) ) == NULL )
 	{
@@ -732,7 +776,7 @@ idl_delete_key (
 					ldbm_cache_delete( db, key );
 
 				} else {
-					SAFEMEMCPY (
+					AC_MEMCPY(
 						&ID_BLOCK_ID(idl, i),
 						&ID_BLOCK_ID(idl, i+1),
 						(ID_BLOCK_NIDS(idl)-i) * sizeof(ID) );
@@ -756,18 +800,17 @@ idl_delete_key (
 	   */
 	for ( nids = 0; !ID_BLOCK_NOID(idl, nids); nids++ )
 		;	/* NULL */
-	kstr = (char *) ch_malloc( key.dsize + 20 );
+
+	cont_alloc( &data, &key );
 
 	for ( j = 0; !ID_BLOCK_NOID(idl, j); j++ ) 
 	{
-		ldbm_datum_init( data );
-		sprintf( kstr, "%c%s%ld", CONT_PREFIX, key.dptr, ID_BLOCK_ID(idl, j) );
-		data.dptr = kstr;
-		data.dsize = strlen( kstr ) + 1;
+		ID_BLOCK *tmp;
+		cont_id( &data, ID_BLOCK_ID(idl, j) );
 
 		if ( (tmp = idl_fetch_one( be, db, data )) == NULL ) {
 			Debug( LDAP_DEBUG_ANY,
-			    "idl_fetch of (%s) returns NULL\n", data.dptr, 0, 0 );
+			    "idl_delete_key: idl_fetch of returned NULL\n", 0, 0, 0 );
 			continue;
 		}
 		/*
@@ -777,7 +820,7 @@ idl_delete_key (
 		{
 			if ( ID_BLOCK_ID(tmp, i) == id )
 			{
-				SAFEMEMCPY(
+				AC_MEMCPY(
 					&ID_BLOCK_ID(tmp, i),
 					&ID_BLOCK_ID(tmp, i+1),
 					(ID_BLOCK_NIDS(tmp)-(i+1)) * sizeof(ID));
@@ -789,7 +832,7 @@ idl_delete_key (
 
 				} else {
 					ldbm_cache_delete( db, data );
-					SAFEMEMCPY(
+					AC_MEMCPY(
 						&ID_BLOCK_ID(idl, j),
 						&ID_BLOCK_ID(idl, j+1),
 						(nids-(j+1)) * sizeof(ID));
@@ -801,14 +844,15 @@ idl_delete_key (
 						idl_store( be, db, key, idl );
 				}
 				idl_free( tmp );
-				free( kstr );
+				cont_free( &data );
 				idl_free( idl );
 				return 0;
 			}
 		}
 		idl_free( tmp );
 	}
-	free( kstr );
+
+	cont_free( &data );
 	idl_free( idl );
 	return -1;
 }
@@ -826,11 +870,15 @@ idl_dup( ID_BLOCK *idl )
 
 	new = idl_alloc( ID_BLOCK_NMAX(idl) );
 
-	SAFEMEMCPY(
+	AC_MEMCPY(
 		(char *) new,
 		(char *) idl,
 		(ID_BLOCK_NMAX(idl) + ID_BLOCK_IDS_OFFSET) * sizeof(ID) );
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(new);
+#endif
+
 	return( new );
 }
 
@@ -868,6 +916,11 @@ idl_intersection(
 
 	n = idl_dup( idl_min( a, b ) );
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(a);
+	idl_check(b);
+#endif
+
 	for ( ni = 0, ai = 0, bi = 0; ai < ID_BLOCK_NIDS(a); ai++ ) {
 		for ( ;
 			bi < ID_BLOCK_NIDS(b) && ID_BLOCK_ID(b, bi) < ID_BLOCK_ID(a, ai);
@@ -891,6 +944,10 @@ idl_intersection(
 	}
 	ID_BLOCK_NIDS(n) = ni;
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(n);
+#endif
+
 	return( n );
 }
 
@@ -918,6 +975,11 @@ idl_union(
 		return( idl_allids( be ) );
 	}
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(a);
+	idl_check(b);
+#endif
+
 	if ( ID_BLOCK_NIDS(b) < ID_BLOCK_NIDS(a) ) {
 		n = a;
 		a = b;
@@ -950,6 +1012,10 @@ idl_union(
 	}
 	ID_BLOCK_NIDS(n) = ni;
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(n);
+#endif
+
 	return( n );
 }
 
@@ -1027,6 +1093,10 @@ idl_notin(
 	}
 	ID_BLOCK_NIDS(n) = ni;
 
+#ifdef LDBM_DEBUG_IDL
+	idl_check(n);
+#endif
+
 	return( n );
 }
 
@@ -1035,10 +1105,14 @@ idl_notin(
  *		NIDS > 1 return 1
  *		otherwise return NOID 
  *	otherwise return first ID
+ *
+ *	cursor is set to 1
  */         
 ID
-idl_firstid( ID_BLOCK *idl )
+idl_firstid( ID_BLOCK *idl, ID *cursor )
 {
+	*cursor = 1;
+
 	if ( idl == NULL || ID_BLOCK_NIDS(idl) == 0 ) {
 		return( NOID );
 	}
@@ -1050,28 +1124,29 @@ idl_firstid( ID_BLOCK *idl )
 	return( ID_BLOCK_ID(idl, 0) );
 }
 
-/*	return next ID after id
- *	if ALLIDS block, increment id. 
+/*	return next ID
+ *	if ALLIDS block, cursor is id.
+ *		increment id
  *		if id < NIDS return id
  *		otherwise NOID.
- *	otherwise SEARCH for next id (ugh!)
+ *	otherwise cursor is index into block
+ *		if index < nids
+ *			return id at index then increment
  */ 
 ID
-idl_nextid( ID_BLOCK *idl, ID id )
+idl_nextid( ID_BLOCK *idl, ID *cursor )
 {
-	unsigned int	i;
-
 	if ( ID_BLOCK_ALLIDS( idl ) ) {
-		return( ++id < ID_BLOCK_NIDS(idl) ? id : NOID );
+		if( ++(*cursor) < ID_BLOCK_NIDS(idl) ) {
+			return *cursor;
+		} else {
+			return NOID;
+		}
 	}
 
-	for ( i = 0; i < ID_BLOCK_NIDS(idl) && ID_BLOCK_ID(idl, i) <= id; i++ ) {
-		;	/* NULL */
+	if ( *cursor < ID_BLOCK_NIDS(idl) ) {
+		return( ID_BLOCK_ID(idl, (*cursor)++) );
 	}
 
-	if ( i >= ID_BLOCK_NIDS(idl) ) {
-		return( NOID );
-	} else {
-		return( ID_BLOCK_ID(idl, i) );
-	}
+	return( NOID );
 }
