@@ -1,4 +1,9 @@
 /* unbind.c - shell backend unbind function */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/unbind.c,v 1.6.2.4 2002/01/04 20:38:35 kurt Exp $ */
+/*
+ * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 
 #include "portable.h"
 
@@ -10,7 +15,7 @@
 #include "slap.h"
 #include "shell.h"
 
-void
+int
 shell_back_unbind(
     Backend		*be,
     Connection		*conn,
@@ -21,25 +26,23 @@ shell_back_unbind(
 	FILE			*rfp, *wfp;
 
 	if ( si->si_unbind == NULL ) {
-		send_ldap_result( conn, op, LDAP_UNWILLING_TO_PERFORM, NULL,
-		    "unbind not implemented" );
-		return;
+		return 0;
 	}
 
 	if ( (op->o_private = (void *) forkandexec( si->si_unbind, &rfp, &wfp ))
 	    == (void *) -1 ) {
-		send_ldap_result( conn, op, LDAP_OPERATIONS_ERROR, NULL,
-		    "could not fork/exec" );
-		return;
+		return 0;
 	}
 
 	/* write out the request to the unbind process */
 	fprintf( wfp, "UNBIND\n" );
-	fprintf( wfp, "msgid: %ld\n", op->o_msgid );
+	fprintf( wfp, "msgid: %ld\n", (long) op->o_msgid );
 	print_suffixes( wfp, be );
 	fprintf( wfp, "dn: %s\n", (conn->c_dn ? conn->c_dn : "") );
 	fclose( wfp );
 
 	/* no response to unbind */
 	fclose( rfp );
+
+	return 0;
 }
